==> IRanges_1.4.16.tar.gz: IRanges/R/DataFrame-utils.R <==
### =========================================================================
### DataFrame utilities
### -------------------------------------------------------------------------

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Splitting and combining.
###

setMethod("split", "DataFrame",
          function(x, f, drop=FALSE) {
            newCompressedList("CompressedSplitDataFrameList", x,
                              splitFactor = f, drop = drop)
          })

setMethod("cbind", "DataFrame",
          function(..., deparse.level=1) {
            ans <- DataFrame(...)
            elementMetadata(ans) <-
              do.call(rbind, lapply(list(...), elementMetadata))
            ans
          })

setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
  args <- list(...)
  hasrows <- unlist(lapply(args, nrow), use.names=FALSE) > 0
  hascols <- unlist(lapply(args, ncol), use.names=FALSE) > 0

  if (!any(hasrows | hascols)) {
    return(DataFrame())
  } else if (!any(hasrows)) {
    return(args[[which(hascols)[1]]])
  } else if (sum(hasrows) == 1) {
    return(args[[which(hasrows)]])
  } else {
    args <- args[hasrows]
  }

  df <- args[[1]]

  for (i in 2:length(args)) {
    if (ncol(df) != ncol(args[[i]]))
      stop("number of columns for arg ", i, " do not match those of first arg")
    if (!identical(colnames(df), colnames(args[[i]])))
      stop("column names for arg ", i, " do not match those of first arg")
  }

  if (ncol(df) == 0) {
    ans <- DataFrame()
    ans@nrows <- sum(unlist(lapply(args, nrow), use.names=FALSE))
  } else {
    cn <- colnames(df)
    cl <- unlist(lapply(as.list(df, use.names = FALSE), class))
    factors <- unlist(lapply(as.list(df, use.names = FALSE), is.factor))
    cols <- lapply(seq_len(length(df)), function(i) {
      cols <- lapply(args, `[[`, cn[i])
      if (factors[i]) { # combine factor levels, coerce to character
        levs <- unique(unlist(lapply(cols, levels), use.names=FALSE))
        cols <- lapply(cols, as.character)
      }
      combined <- do.call(c, cols)
      if (factors[i])
        combined <- factor(combined, levs)
      ## this coercion needed only because we extracted ([[) above
      ## which brings external -> internal
      ## external objects should support external combination (c)
      as(combined, cl[i])
    })
    names(cols) <- colnames(df)
    ans <- do.call(DataFrame, cols)
  }

  rn <- unlist(lapply(args, rownames), use.names=FALSE)
  if (!is.null(rn)) {
    if (length(rn) != nrow(ans)) {
      rn <- NULL
    } else if (anyDuplicated(rn))
      rn <- make.unique(rn, sep = "")
  }
  rownames(ans) <- rn

  ans
})

==> IRanges_1.6.17.tar.gz: IRanges/R/DataFrame-utils.R <==
### =========================================================================
### DataFrame utilities
### -------------------------------------------------------------------------

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Splitting and combining.
###

setMethod("split", "DataFrame",
          function(x, f, drop=FALSE) {
            newCompressedList("CompressedSplitDataFrameList", x,
                              splitFactor = f, drop = drop)
          })

setMethod("cbind", "DataFrame",
          function(..., deparse.level=1) {
            ans <- DataFrame(...)
            elementMetadata(ans) <- .rbind.elementMetadata(...)
            ans
          })

setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
  args <- list(...)
  hasrows <- unlist(lapply(args, nrow), use.names=FALSE) > 0L
  hascols <- unlist(lapply(args, ncol), use.names=FALSE) > 0L

  if (!any(hasrows | hascols)) {
    return(DataFrame())
  } else if (!any(hasrows)) {
    return(args[[which(hascols)[1L]]])
  } else if (sum(hasrows) == 1) {
    return(args[[which(hasrows)]])
  } else {
    args <- args[hasrows]
  }

  df <- args[[1L]]

  for (i in 2:length(args)) {
    if (ncol(df) != ncol(args[[i]]))
      stop("number of columns for arg ", i, " do not match those of first arg")
    if (!identical(colnames(df), colnames(args[[i]])))
      stop("column names for arg ", i, " do not match those of first arg")
  }

  if (ncol(df) == 0) {
    ans <- DataFrame()
    ans@nrows <- sum(unlist(lapply(args, nrow), use.names=FALSE))
  } else {
    cn <- colnames(df)
    cl <- unlist(lapply(as.list(df, use.names = FALSE), class))
    factors <- unlist(lapply(as.list(df, use.names = FALSE), is.factor))
    cols <- lapply(seq_len(length(df)), function(i) {
      cols <- lapply(args, `[[`, cn[i])
      if (factors[i]) { # combine factor levels, coerce to character
        levs <- unique(unlist(lapply(cols, levels), use.names=FALSE))
        cols <- lapply(cols, as.character)
      }
      combined <- do.call(c, unname(cols))
      if (factors[i])
        combined <- factor(combined, levs)
      ## this coercion needed only because we extracted ([[) above
      ## which brings external -> internal
      ## external objects should support external combination (c)
      as(combined, cl[i])
    })
    names(cols) <- colnames(df)
    ans <- do.call(DataFrame, cols)
  }

  rn <- unlist(lapply(args, rownames), use.names=FALSE)
  if (!is.null(rn)) {
    if (length(rn) != nrow(ans)) {
      rn <- NULL
    } else if (anyDuplicated(rn))
      rn <- make.unique(rn, sep = "")
  }
  rownames(ans) <- rn

  if (!is.null(elementMetadata(df))) {
    emd <- elementMetadata(df)
    if (all(sapply(args, function(x) identical(elementMetadata(x), emd))))
      elementMetadata(ans) <- emd
  }
  
  ans
})
